import pytest
from cuenca_validations.types import KYCFileType

from cuenca import File


@pytest.mark.vcr
def test_file_upload(file):
    uploaded_file: File = File.upload(
        file=file, file_type=KYCFileType.ine, extension='jpeg'
    )
    assert uploaded_file.id is not None
    assert uploaded_file.type == KYCFileType.ine
    assert uploaded_file.url is not None
    assert uploaded_file.user_id is not None


@pytest.mark.vcr
def test_file_download():
    file: File = File.first(type=KYCFileType.ine)
    assert isinstance(file.file, bytes)


@pytest.mark.vcr
def test_file_error_on_xml_pdf():
    file: File = File.first(type=KYCFileType.ine)
    with pytest.raises(NotImplementedError):
        file.xml

    with pytest.raises(NotImplementedError):
        file.pdf



import datetime as dt
from typing import ClassVar, Optional, cast

from cuenca_validations.types.requests import (
    UserCredentialRequest,
    UserCredentialUpdateRequest,
)
from pydantic.dataclasses import dataclass

from ..http import Session, session as global_session
from .base import Creatable, Updateable


@dataclass
class UserCredential(Creatable, Updateable):
    _resource: ClassVar = 'user_credentials'

    is_active: bool
    created_at: dt.datetime

    @classmethod
    def create(
        cls, password: str, *, session: Session = global_session
    ) -> 'UserCredential':
        req = UserCredentialRequest(password=password)
        return cast(
            'UserCredential', cls._create(**req.dict(), session=session)
        )

    @classmethod
    def update(
        cls,
        user_id: str = 'me',
        is_active: Optional[bool] = None,
        password: Optional[str] = None,
        *,
        session: Session = global_session,
    ) -> 'UserCredential':
        req = UserCredentialUpdateRequest(
            is_active=is_active,
            password=password,
        )
        return cast(
            'UserCredential',
            cls._update(id=user_id, **req.dict(), session=session),
        )



import datetime as dt

import pytest
from cuenca_validations.types import (
    EntryType,
    SavingCategory,
    TransactionStatus,
    WalletTransactionType,
)

from cuenca import BalanceEntry, Saving, WalletTransaction


@pytest.mark.vcr
def test_create_wallet_transaction():
    wallet_id = 'LAvWUDH6OpQk-ber3E_zUEiQ'
    deposit = WalletTransaction.create(
        wallet_uri=f'/savings/{wallet_id}',
        transaction_type=WalletTransactionType.deposit,
        amount=10000,
    )
    assert deposit.id is not None
    assert deposit.transaction_type == WalletTransactionType.deposit
    assert deposit.status == TransactionStatus.succeeded
    wallet = deposit.wallet
    assert wallet.id == wallet_id


@pytest.mark.vcr
def test_retrieve_wallet_transaction():
    id = 'LT32GEaFQR03cJRBcqb0p7uI'
    transaction = WalletTransaction.retrieve(id)
    assert transaction.id == id
    assert transaction.status == TransactionStatus.succeeded


@pytest.mark.vcr
def test_query_wallet_transactions():
    wallet_uri = '/savings/LAGdf-FVVeQeeKrmYpF5NIfA'
    query = WalletTransaction.all(wallet_uri=wallet_uri)
    transactions = [txn for txn in query]
    assert len(transactions) == 2


@pytest.mark.vcr
def test_complete_flow_wallets():
    # create wallet
    saving = Saving.create(
        name='Ahorros',
        category=SavingCategory.travel,
        goal_amount=1000000,
        goal_date=dt.datetime.now() + dt.timedelta(days=365),
    )
    assert saving.balance == 0
    assert saving.wallet_uri == f'/savings/{saving.id}'

    # deposit money in wallet
    deposit = WalletTransaction.create(
        wallet_uri=saving.wallet_uri,
        transaction_type=WalletTransactionType.deposit,
        amount=10000,
    )
    assert deposit.status == TransactionStatus.succeeded
    saving.refresh()
    assert saving.balance == deposit.amount
    deposit_uri = f'/wallet_transactions/{deposit.id}'

    # withdraw money from wallet
    withdrawal = WalletTransaction.create(
        wallet_uri=saving.wallet_uri,
        transaction_type=WalletTransactionType.withdrawal,
        amount=2000,
    )
    assert withdrawal.status == TransactionStatus.succeeded
    saving.refresh()
    assert saving.balance == deposit.amount - withdrawal.amount
    withdrawal_uri = f'/wallet_transactions/{withdrawal.id}'

    # Check all transactions was created
    query = WalletTransaction.all(wallet_uri=saving.wallet_uri)
    transactions_db = [wt.id for wt in query]
    assert deposit.id in transactions_db
    assert withdrawal.id in transactions_db

    # check balance entries created for wallet
    entries = BalanceEntry.all(wallet_id=saving.id)
    wallet_entries = [entry for entry in entries]
    assert len(wallet_entries) == 2
    # default -> deposit -> wallet  (credit in wallet)
    credit = [be for be in wallet_entries if be.type == EntryType.credit][0]
    assert credit.related_transaction_uri == deposit_uri
    assert credit.amount == deposit.amount
    # default <- withdrawal <- wallet (debit in wallet)
    debit = [be for be in wallet_entries if be.type == EntryType.debit][0]
    assert debit.amount == withdrawal.amount
    assert debit.related_transaction_uri == withdrawal_uri

    # check balance entries created in default, related with wallet
    entries = BalanceEntry.all(
        wallet_id='default', funding_instrument_uri=saving.wallet_uri
    )
    default_entries = [entry for entry in entries]
    assert len(default_entries) == 2
    # default -> deposit -> wallet  (debit in default)
    debit = [be for be in default_entries if be.type == EntryType.debit][0]
    assert debit.related_transaction_uri == deposit_uri
    assert debit.amount == deposit.amount
    # default <- withdrawal <- wallet (credit in default)
    credit = [be for be in default_entries if be.type == EntryType.credit][0]
    assert credit.amount == withdrawal.amount
    assert credit.related_transaction_uri == withdrawal_uri



import pytest
from cuenca_validations.types import TransactionStatus, TransferNetwork

from cuenca import WhatsappTransfer


@pytest.mark.vcr
def test_wa_transfer_cuenca():
    id_wa = 'SW01'
    wa_transfer: WhatsappTransfer = WhatsappTransfer.retrieve(id_wa)
    assert wa_transfer.id == id_wa
    assert wa_transfer.network == TransferNetwork.internal
    account = wa_transfer.destination
    assert account is not None
    assert account.id.startswith('LA')


@pytest.mark.vcr
def test_wa_transfer_spei():
    id_wa = 'SW03'
    wa_transfer: WhatsappTransfer = WhatsappTransfer.retrieve(id_wa)
    assert wa_transfer.id == id_wa
    assert wa_transfer.network == TransferNetwork.spei
    assert wa_transfer.destination_uri


@pytest.mark.vcr
def test_wa_transfer_submitted():
    id_wa = 'SW02'
    wa_transfer: WhatsappTransfer = WhatsappTransfer.retrieve(id_wa)
    assert wa_transfer.id == id_wa
    assert wa_transfer.status == TransactionStatus.submitted
    account = wa_transfer.destination
    assert account is None



import pytest

from cuenca.resources import Arpc


@pytest.mark.vcr
def test_arpc():
    arpc_req = dict(
        number='1234567890123403',
        arqc='DB3C77D5469C53C6',
        arpc_method='1',
        transaction_data='somerandomtransactiondata',
        response_code='0010',
        pan_sequence='01',
        unique_number='42D6A016',
        transaction_counter='001D',
        track_data_method='terminal',
    )
    arpc = Arpc.create(**arpc_req)
    assert arpc.is_valid_arqc



import datetime as dt
from unittest.mock import patch

import pytest
from cuenca_validations.errors import (
    NoPasswordFoundError,
    UserNotLoggedInError,
)
from freezegun import freeze_time

from cuenca.exc import CuencaResponseException
from cuenca.http.client import Session
from cuenca.resources import Card, Transfer, UserCredential, UserLogin


@pytest.mark.vcr
def test_invalid_auth():
    session = Session()
    session.configure(sandbox=False)
    with pytest.raises(CuencaResponseException) as e:
        session.post('/api_keys', dict())
    assert e.value.status_code == 401
    assert str(e.value)


@pytest.mark.usefixtures('cuenca_creds')
def test_basic_auth_configuration():
    session = Session()
    assert session.auth == session.basic_auth
    assert session.auth == ('api_key', 'secret')
    assert not session.jwt_token


@pytest.mark.vcr
@pytest.mark.usefixtures('cuenca_creds')
def test_configures_jwt():
    session = Session()
    session.configure(use_jwt=True)
    assert session.auth
    assert session.jwt_token


@pytest.mark.vcr
@pytest.mark.usefixtures('cuenca_creds')
def test_request_valid_token():
    session = Session()
    # Set date when the cassette was created otherwise will retrieve
    # an expired token
    with freeze_time(dt.date(2020, 10, 22)):
        session.configure(use_jwt=True)
        response = session.get('/api_keys')
    assert response['items']


@pytest.mark.vcr
@pytest.mark.usefixtures('cuenca_creds')
def test_request_expired_token():
    session = Session()
    session.configure(use_jwt=True)
    previous_jwt = session.jwt_token.token
    with freeze_time(dt.datetime.utcnow() + dt.timedelta(days=40)):
        response = session.get('/api_keys')
    assert response['items']
    assert session.jwt_token != previous_jwt


@patch('cuenca.http.client.requests.Session.request')
def test_overrides_session(mock_request):
    mock_request.return_value.ok = True
    mock_request.return_value.content = '{"items": []}'
    session = Session()
    session.configure(
        api_key='USER_API_KEY', api_secret='USER_SECRET', sandbox=True
    )
    Card.first(user_id='USER_ID', session=session)
    mock_request.assert_called_once()
    _, kwargs = mock_request.call_args_list[0]
    assert kwargs['auth'] == session.auth


@pytest.mark.vcr
def test_no_password():
    UserLogin.create('111111')
    UserCredential.update(password=None)
    with pytest.raises(NoPasswordFoundError):
        Transfer.count()


@pytest.mark.vcr
def test_no_session():
    with pytest.raises(UserNotLoggedInError):
        Transfer.count()



import base64
import datetime as dt
import json
from dataclasses import asdict, dataclass, fields
from io import BytesIO
from typing import ClassVar, Dict, Generator, Optional, Union
from urllib.parse import urlencode

from cuenca_validations.types import (
    FileFormat,
    QueryParams,
    SantizedDict,
    TransactionQuery,
    TransactionStatus,
)

from ..exc import MultipleResultsFound, NoResultFound
from ..http import Session, session as global_session


@dataclass
class Resource:
    _resource: ClassVar[str]

    id: str

    # purely for MyPy
    def __init__(self, **_):  # pragma: no cover
        ...

    @classmethod
    def _from_dict(cls, obj_dict: Dict[str, Union[str, int]]) -> 'Resource':
        cls._filter_excess_fields(obj_dict)
        return cls(**obj_dict)

    @classmethod
    def _filter_excess_fields(cls, obj_dict):
        """
        dataclasses don't allow __init__ to be called with excess fields. This
        method allows the API to add fields in the response body without
        breaking the client
        """
        excess = set(obj_dict.keys()) - {f.name for f in fields(cls)}
        for f in excess:
            del obj_dict[f]

    def to_dict(self):
        return asdict(self, dict_factory=SantizedDict)


class Retrievable(Resource):
    @classmethod
    def retrieve(
        cls, id: str, *, session: Session = global_session
    ) -> Resource:
        resp = session.get(f'/{cls._resource}/{id}')
        return cls._from_dict(resp)

    def refresh(self, *, session: Session = global_session):
        new = self.retrieve(self.id, session=session)
        for attr, value in new.__dict__.items():
            setattr(self, attr, value)


class Creatable(Resource):
    @classmethod
    def _create(cls, *, session: Session = global_session, **data) -> Resource:
        resp = session.post(cls._resource, data)
        return cls._from_dict(resp)


@dataclass
class Updateable(Resource):
    updated_at: dt.datetime

    @classmethod
    def _update(
        cls, id: str, *, session: Session = global_session, **data
    ) -> Resource:
        resp = session.patch(f'/{cls._resource}/{id}', data)
        return cls._from_dict(resp)


@dataclass
class Deactivable(Resource):
    deactivated_at: Optional[dt.datetime]

    @classmethod
    def deactivate(
        cls, id: str, *, session: Session = global_session, **data
    ) -> Resource:
        resp = session.delete(f'/{cls._resource}/{id}', data)
        return cls._from_dict(resp)

    @property
    def is_active(self):
        return not self.deactivated_at


@dataclass
class Downloadable(Resource):
    @classmethod
    def download(
        cls,
        instance,
        file_format: FileFormat,
        *,
        session: Session = global_session,
    ) -> BytesIO:
        resp = session.request(
            'get',
            f'/{cls._resource}/{instance.id}',
            headers=dict(Accept=file_format.value),
        )
        return BytesIO(resp)

    @property
    def pdf(self) -> bytes:
        return self.download(self, file_format=FileFormat.pdf).read()

    @property
    def xml(self) -> bytes:
        return self.download(self, file_format=FileFormat.xml).read()


@dataclass
class Uploadable(Resource):
    @classmethod
    def _upload(
        cls,
        file: bytes,
        user_id: str,
        *,
        session: Session = global_session,
        **data,
    ) -> Resource:
        encoded_file = base64.b64encode(file)
        resp = session.request(
            'post',
            cls._resource,
            files=dict(
                file=(None, encoded_file),
                user_id=(None, user_id),
                **{k: (None, v) for k, v in data.items()},
            ),
        )
        return cls._from_dict(json.loads(resp))


@dataclass
class Queryable(Resource):
    _query_params: ClassVar = QueryParams

    created_at: dt.datetime

    @classmethod
    def one(
        cls, *, session: Session = global_session, **query_params
    ) -> Resource:
        q = cls._query_params(limit=2, **query_params)
        resp = session.get(cls._resource, q.dict())
        items = resp['items']
        len_items = len(items)
        if not len_items:
            raise NoResultFound
        if len_items > 1:
            raise MultipleResultsFound
        return cls._from_dict(items[0])

    @classmethod
    def first(
        cls, *, session: Session = global_session, **query_params
    ) -> Optional[Resource]:
        q = cls._query_params(limit=1, **query_params)
        resp = session.get(cls._resource, q.dict())
        try:
            item = resp['items'][0]
        except IndexError:
            rv = None
        else:
            rv = cls._from_dict(item)
        return rv

    @classmethod
    def count(
        cls, *, session: Session = global_session, **query_params
    ) -> int:
        q = cls._query_params(count=True, **query_params)
        resp = session.get(cls._resource, q.dict())
        return resp['count']

    @classmethod
    def all(
        cls, *, session: Session = global_session, **query_params
    ) -> Generator[Resource, None, None]:
        session = session or global_session
        q = cls._query_params(**query_params)
        next_page_uri = f'{cls._resource}?{urlencode(q.dict())}'
        while next_page_uri:
            page = session.get(next_page_uri)
            yield from (cls._from_dict(item) for item in page['items'])
            next_page_uri = page['next_page_uri']


@dataclass
class Transaction(Retrievable, Queryable):
    _query_params: ClassVar = TransactionQuery

    user_id: str
    amount: int  # in centavos
    status: TransactionStatus
    descriptor: str  # how it appears for the customer


@dataclass
class Wallet(Creatable, Deactivable, Retrievable, Queryable):
    user_id: str
    balance: int

    @property
    def wallet_uri(self):
        return f'/{self._resource}/{self.id}'



import pytest

from cuenca import Commission, Deposit, Transfer


@pytest.mark.vcr
def test_commission_retrieve():
    id_commission = 'COXXX'
    commission: Commission = Commission.retrieve(id_commission)
    assert commission.id == id_commission


@pytest.mark.vcr
def test_commission_retrieve_with_cash_deposit():
    id_commission = 'COXXX'
    commission: Commission = Commission.retrieve(id_commission)
    assert commission.id == id_commission
    related_transaction = commission.related_transaction
    assert related_transaction
    assert type(related_transaction) == Deposit
    assert related_transaction.network == 'cash'


@pytest.mark.vcr
def test_commission_retrieve_with_cash_transfer():
    id_commission = 'COXXX'
    commission: Commission = Commission.retrieve(id_commission)
    assert commission.id == id_commission
    related_transaction = commission.related_transaction
    assert related_transaction
    assert type(related_transaction) == Transfer
    assert related_transaction.network == 'spei'



import datetime as dt

import pytest
from cuenca_validations.types import SavingCategory

from cuenca import Saving


@pytest.mark.vcr
def test_saving_create():
    saving = Saving.create(
        name='my new car',
        category=SavingCategory.vehicle,
        goal_amount=100000,
        goal_date=(dt.datetime.utcnow() + dt.timedelta(days=365)),
    )
    assert saving.id is not None
    assert saving.is_active
    assert saving.balance == 0


@pytest.mark.vcr
def test_saving_retrieve():
    saving_id = 'lAob5rOC0jSj6UC5RAbwnSnA'
    saving = Saving.retrieve(saving_id)
    assert saving.id == saving_id


@pytest.mark.vcr
def test_saving_update():
    saving_id = 'lAob5rOC0jSj6UC5RAbwnSnA'
    changes = dict(
        name='my new home',
        goal_amount=200000,
        category=SavingCategory.home,
    )
    saving = Saving.update(saving_id, **changes)
    assert all(item in saving.to_dict().items() for item in changes.items())


@pytest.mark.vcr
def test_saving_deactivate():
    saving_id = 'lAob5rOC0jSj6UC5RAbwnSnA'
    saving = Saving.deactivate(saving_id)
    assert saving.deactivated_at is not None



from typing import ClassVar

from pydantic.dataclasses import dataclass

from .identity_events import IdentityEvent


@dataclass
class UserEvent(IdentityEvent):
    _resource: ClassVar = 'user_events'

    user_id: str
    platform_id: str



import pytest

from cuenca import Webhook


@pytest.mark.vcr
def test_webhook_retrieve():
    id_webhook = 'WE01'
    webhook: Webhook = Webhook.retrieve(id_webhook)
    assert webhook.id == id_webhook
    assert webhook.payload
    assert webhook.event


@pytest.mark.vcr
def test_webhook_all():
    assert list(Webhook.all())



from typing import ClassVar, List, Optional, cast

from cuenca_validations.types.enums import WebhookEvent
from cuenca_validations.types.requests import (
    EndpointRequest,
    EndpointUpdateRequest,
)
from pydantic import HttpUrl
from pydantic.dataclasses import dataclass

from ..http import Session, session as global_session
from .base import Creatable, Deactivable, Queryable, Retrievable, Updateable


@dataclass()
class Endpoint(Creatable, Deactivable, Retrievable, Queryable, Updateable):
    _resource: ClassVar = 'endpoints'

    url: HttpUrl
    secret: str
    is_enable: bool
    events: List[WebhookEvent]

    @classmethod
    def create(
        cls,
        url: HttpUrl,
        events: Optional[List[WebhookEvent]] = None,
        *,
        session: Session = global_session,
    ) -> 'Endpoint':
        """
        Creates and Endpoint, allowing to recieve Webhooks with the specified
        events.

        :param url: HTTPS url to send webhooks
        :param events: list of enabled events. If None, all events will be
            enabled for this Endpoint
        :param session:
        :return: New active endpoint
        """
        req = EndpointRequest(url=url, events=events)
        return cast('Endpoint', cls._create(session=session, **req.dict()))

    @classmethod
    def update(
        cls,
        endpoint_id: str,
        url: Optional[HttpUrl] = None,
        is_enable: Optional[bool] = None,
        *,
        session: Session = global_session,
    ) -> 'Endpoint':
        """
        Updates endpoint properties. It allows reconfigure properties
        like url and is_active.

        :param endpoint_id: existing endpoint_id
        :param url
        :param is_enable
        :param session
        :return: Updated endpoint object
        """
        req = EndpointUpdateRequest(url=url, is_enable=is_enable)
        resp = cls._update(endpoint_id, session=session, **req.dict())
        return cast('Endpoint', resp)



import pytest

from cuenca import UserLogin
from cuenca.http.client import Session


@pytest.fixture
def session():
    session = Session()
    session.configure(
        'api_key',
        'api_secret',
        sandbox=True,
    )
    return session


@pytest.mark.vcr
def test_valid_login(session: Session):
    login = UserLogin.create('222222', session=session)
    assert login.success
    assert login.last_login_at is not None
    assert login.id is not None
    assert session.session.headers['X-Cuenca-LoginId'] == login.id


@pytest.mark.vcr
def test_invalid_login(session: Session):
    login = UserLogin.create('111111', session=session)
    assert not login.success
    assert login.last_login_at is None
    assert login.id is not None
    assert 'X-Cuenca-LoginId' not in session.session.headers


@pytest.mark.vcr
def test_logout(session: Session):
    UserLogin.create('222222', session=session)
    UserLogin.logout(session=session)
    assert 'X-Cuenca-LoginId' not in session.session.headers



import pytest
from cuenca_validations.types import TransactionStatus, TransferNetwork
from pydantic import ValidationError

from cuenca import Transfer
from cuenca.exc import MultipleResultsFound, NoResultFound
from cuenca.resources.transfers import TransferRequest


@pytest.mark.vcr
def test_transfers_create():
    transfer = Transfer.create(
        account_number='646180157034181180',
        amount=10000,
        descriptor='Mi primer transferencia',
        recipient_name='Rogelio Lopez',
    )
    assert transfer.id is not None
    assert transfer.idempotency_key is not None
    assert transfer.status
    assert transfer.status == TransactionStatus.submitted
    assert transfer.network == TransferNetwork.internal
    account = transfer.destination
    assert account.name == 'Rogelio Lopez'
    assert account.account_number == '646180157034181180'
    assert account.institution_name == 'Cuenca'
    # Some seconds later
    transfer.refresh()
    assert transfer.status == TransactionStatus.succeeded


@pytest.mark.vcr
def test_transfers_create_many():
    valid = [
        TransferRequest(
            account_number='646180157034181180',
            amount=10000,
            descriptor='Mi primer transferencia',
            recipient_name='Rogelio Lopez',
            idempotency_key='35b241e25814445faf25c9cbcfc388a6',
        ),
        TransferRequest(
            account_number='646180157034181180',
            amount=10001,
            descriptor='Mi segundo transferencia',
            recipient_name='Rogelio Lopez',
            idempotency_key='dc15fc432a734724ab1e5884a4a24a2c',
        ),
    ]
    invalid = [
        TransferRequest(
            # BIN doesn't belong to any MX banks
            account_number='4050000000000001',
            amount=10002,
            descriptor='Mi transferencia invalida',
            recipient_name='Rogelio Lopez',
            idempotency_key='4a92e77054ba4e369a134e400f7c313e',
        ),
    ]
    transfers = Transfer.create_many(valid + invalid)
    assert {req.idempotency_key for req in valid} == {
        tr.idempotency_key for tr in transfers['submitted']
    }
    assert all([tr.status == 'submitted' for tr in transfers['submitted']])
    assert len(transfers['errors']) == 1
    assert transfers['errors'][0]['request'] == invalid[0]
    # TODO (for Glen): add validation of the exception


@pytest.mark.vcr
def test_transfers_retrieve():
    id_transfer = 'test'
    transfer: Transfer = Transfer.retrieve(id_transfer)
    assert transfer.id == id_transfer
    assert transfer.status is not None


@pytest.mark.vcr
def test_transfers_one():
    key = 'idempotency_key_1'
    transfer: Transfer = Transfer.one(idempotency_key=key)
    assert transfer.idempotency_key == key


@pytest.mark.vcr
def test_transfers_one_errors():
    with pytest.raises(NoResultFound):
        Transfer.one(idempotency_key='wrong_key')

    with pytest.raises(MultipleResultsFound):
        Transfer.one(status=TransactionStatus.submitted)


@pytest.mark.vcr
def test_transfers_first():
    account = '646180157046685645'
    transfer = Transfer.first(account_number=account)
    assert transfer is not None
    assert transfer.account_number == account
    transfer = Transfer.first(account_number='bad_account')
    assert transfer is None


@pytest.mark.vcr
def test_transfers_all():
    transfers = Transfer.all(status=TransactionStatus.succeeded)
    assert all([tr.status is TransactionStatus.succeeded for tr in transfers])


@pytest.mark.vcr
def test_transfers_count():
    # Count all items
    count = Transfer.count()
    assert count == 42

    # Count with filters
    count = Transfer.count(status=TransactionStatus.succeeded)
    assert count == 4


@pytest.mark.vcr
def test_transfers_count_vs_all():
    assert Transfer.count(status=TransactionStatus.succeeded) == len(
        list(Transfer.all(status=TransactionStatus.succeeded))
    )
    assert Transfer.count() == len(list(Transfer.all()))


def test_invalid_params():
    with pytest.raises(ValidationError) as e:
        Transfer.one(invalid_param='invalid_param')
    assert 'extra fields not permitted' in str(e)



import pytest
from cuenca_validations.types import EventType

from cuenca import IdentityEvent


@pytest.mark.vcr
def test_identity_event_retrieve():
    identity_id = 'IDKWLi_wUoTty6WXXg9xfKFQ'
    identity_event = IdentityEvent.one(
        identity_id=identity_id, type=EventType.created.value
    )
    assert identity_event.id is not None



import pytest

from cuenca import BalanceEntry


@pytest.mark.vcr
def test_balance_entry_retrieve():
    id_entry = 'LE123'
    balance_entry: BalanceEntry = BalanceEntry.retrieve(id_entry)
    assert balance_entry.id == id_entry
    assert balance_entry.related_transaction
    assert balance_entry.funding_instrument
    assert balance_entry.wallet_id == 'default'


@pytest.mark.vcr
def test_query_balance_entry():
    funding_instrument_uri = '/accounts/AC123'
    balance_entry: BalanceEntry = BalanceEntry.one(
        funding_instrument_uri=funding_instrument_uri
    )
    balance_entry.funding_instrument_uri = funding_instrument_uri
